import pytest


def pytest_addoption(parser):
    parser.addoption("--project_id", action="store", default="default project_id")
    parser.addoption("--secret", action="store", default="default secret")
    parser.addoption("--email", action="store", default="default email")
    parser.addoption("--phone_number", action="store", default="default phone number")


@pytest.fixture(scope="session")
def project_id(request):
    return request.config.getoption("--project_id")


@pytest.fixture(scope="session")
def secret(request):
    return request.config.getoption("--secret")


@pytest.fixture(scope="session")
def email(request):
    return request.config.getoption("--email")


@pytest.fixture(scope="session")
def phone_number(request):
    return request.config.getoption("--phone_number")



class StytchError(Exception):
    def __init__(
        self,
        request_id: str = None,
        error_type: str = None,
        error_message: str = None,
        error_url: str = None,
        **kwargs
    ):
        self.request_id = request_id
        self.error_type = error_type
        self.error_message = error_message
        self.error_url = error_url

    def __repr__(self):
        return "StytchError {0}".format(self.__dict__)

    def __str__(self):
        return str(self.__dict__)



import json
import requests
from stytch.version import __version__

from typing import Dict, Set

from .decorator import throw_stytch_exception

def _validate_attributes(attributes: Dict[str, str]) -> Dict[str, str]:
    if not attributes:
        return attributes
    default_attributes = {}

    if attributes.get("ip_address"):
        default_attributes.update({"ip_address": attributes["ip_address"]})
    if attributes.get("user_agent"):
        default_attributes.update({"user_agent": attributes["user_agent"]})

    return default_attributes

class Base:
    def __init__(self, client):
        self._requester_base = requests
        self.headers = {
            'Content-Type': 'application/json',
            'User-Agent': 'Stytch Python v{}'.format(__version__)
        }
        self.client = client
        self.auth = requests.auth.HTTPBasicAuth(
            self.client.project_id, self.client.secret
        )

    def _validate_options(self, options: Dict[str, bool]) -> Dict[str, bool]:
        if not options:
            return options

        default_options = {}
        if options.get("ip_match_required"):
            default_options.update(
                {"ip_match_required": options["ip_match_required"]}
            )
        if options.get("user_agent_match_required"):
            default_options.update(
                {"user_agent_match_required": options["user_agent_match_required"]}
            )

        return default_options

    def _validate_fields(self, accepted_fields: Set[str], fields: Set[str]) -> bool:
        if len(accepted_fields.union(fields)) > len(accepted_fields):
            raise Exception("Unknown arguments applied")

        return True

    def get_url(self, arg: str):
        return "{0}{1}".format(self.client.base_url, arg)

    @throw_stytch_exception
    def _get(self, url: str, query_params: Dict = {}):
        return self._requester_base.get(url, auth=self.auth, headers=self.headers, params=query_params)

    @throw_stytch_exception
    def _post(self, url: str, data: Dict):
        return self._requester_base.post(url, auth=self.auth, headers=self.headers, data=json.dumps(data))

    @throw_stytch_exception
    def _put(self, url: str, data: Dict):
        return self._requester_base.put(url, auth=self.auth, headers=self.headers, data=json.dumps(data))

    @throw_stytch_exception
    def _delete(self, url: str):
        return self._requester_base.delete(url, auth=self.auth, headers=self.headers)



import json
import requests

from .error import StytchError


def throw_stytch_exception(func):
    def wrapper(*args, **kwargs):
        resp: requests.models.Response = func(*args, **kwargs)
        if resp.status_code == 401:
            raise PermissionError("Invalid project id or secret")

        if resp.status_code == 400:
            raise StytchError(**json.loads(resp._content))

        else:
            return resp

    return wrapper
